/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_ELEMENT_TRANSFORM_HPP
#define DATAFRAME_ELEMENT_TRANSFORM_HPP 1

#include <type_traits>
#include <boost/lexical_cast.hpp>

namespace dataframe
{

  template < typename T
	     , typename U = T
	     , typename = typename std::is_convertible<T,U>::type
	     >
  class element_transform;

  template < typename T >
  struct element_transform < T,T,std::true_type >
  {
    auto operator () (T t) const { return apply(t); }
    static auto apply (T t) { return t; }
  };

  template < typename T, typename U >
  struct element_transform< T,U,std::true_type >
  {
    auto operator () (T t) const { return apply(t); }
    static auto apply (T t) { return U(t); }
  };

  template < typename T, typename U >
  struct element_transform< T,U,std::false_type >
  {
    auto operator () (T t) const { return apply(t); }
    static auto apply (T t)
    {
      using boost::lexical_cast;
      return lexical_cast<U>(t);
    }
  };

} // namespace dataframe

#endif
