/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_GRAPH_ADJACENCY_GRAPH_BASE_HPP
#define GRAPH_MODEL_GRAPH_ADJACENCY_GRAPH_BASE_HPP 1

#include <boost/graph/adjacency_iterator.hpp>
#include <graph_model/graph/graph_base.hpp>
#include <graph_model/graph_tags.hpp>
#include <graph_model/traits/vertex_descriptor.hpp>

namespace graph_model
{

  namespace graph
  {

    template < typename Derived, typename Base = graph_base >
    class adjacency_graph_base
      : public Base
    {
      using vertex_descriptor = traits::vertex_descriptor_t<::graph_model::tag::adjacency_graph>;

    public:
      adjacency_graph_base () = default;

      template < typename Graph >
      adjacency_graph_base (Graph const& g, vertex_descriptor source)
	: Base(g,source)
	, source_(source)
      {}

      auto source () const { return source_; }

    private:
      vertex_descriptor source_;
    };

    template < typename VertexDescriptor, typename Graph >
    auto adjacent_vertices (VertexDescriptor u, Graph const& graph)
    {
      using graph_type = Graph;
      using vertex_descriptor = VertexDescriptor;

      auto out_edges_range = out_edges(u,graph);
      using out_edge_iterator = decltype(out_edges_range.first);

      using adjacency_iterator = typename boost::adjacency_iterator_generator<graph_type,vertex_descriptor,out_edge_iterator>::type;

      return
	std::make_pair(adjacency_iterator(out_edges_range.first,&graph),
		       adjacency_iterator(out_edges_range.second,&graph));
    }

  } // namespace graph

} // namespace graph_model

#endif
