/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_VECTOR_MPL_VECTOR_BASE_HPP
#define SAMPLE_SPACE_VECTOR_MPL_VECTOR_BASE_HPP 1

#include <sample_space/vector/mpl/basis.hpp>
#include <sample_space/vector/mpl/storage.hpp>
#include <sample_space/vector/policy/euclidean_vector_space_base.hpp>
#include <sample_space/vector/policy/simplex_base.hpp>
#include <use_default.hpp>

namespace sample_space
{

  namespace vector
  {

    namespace mpl
    {
    
      template < typename Derived
		 , typename T
		 , typename Index
		 , typename Tag
		 , typename Basis
		 , typename Storage
		 >
      class vector_base;

      template < typename Derived
		 , typename T
		 , typename Index
		 , typename Basis
		 , typename Storage
		 >
      struct vector_base<Derived,T,Index,tag::euclidean_vector_space,Basis,Storage>
      {
	using type
	= policy::euclidean_vector_space_base<
	  Derived
	  , T
	  , Index
	  , tag::euclidean_vector_space
	  , mpl::basis_t<tag::euclidean_vector_space,Basis>
	  , mpl::storage_t<Storage>
	  >;
      };

      template < typename Derived
		 , typename T
		 , typename Index
		 , typename Basis
		 , typename Storage
		 >
      struct vector_base<Derived,T,Index,tag::simplex,Basis,Storage>
      {
	using type
	= policy::simplex_base<
	  Derived
	  , T
	  , Index
	  , tag::simplex
	  , mpl::basis_t<tag::simplex,Basis>
	  , mpl::storage_t<Storage>
	  >;
      };

      template < typename Derived
		 , typename T
		 , typename Index
		 , typename Basis
		 , typename Storage
		 >
      struct vector_base<Derived,T,Index,use_default<>,Basis,Storage>
	: vector_base<Derived,T,Index,tag::euclidean_vector_space,Basis,Storage>
      {};

      template < typename Derived
		 , typename T
		 , typename Index
		 , typename Tag
		 , typename Basis
		 , typename Storage
		 >
      using vector_base_t = typename vector_base<Derived,T,Index,Tag,Basis,Storage>::type;

    } // namespace mpl

  } // namespace vector
  
} // namespace sample_space

#endif
