/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE dataframe_element_transform

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include "dataframe/element_transform.hpp"
#include "dataframe/element_transform_iterator.hpp"
#include <string>
#include <vector>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector<int,long,long long,float,double,long double>;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_typedef,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T>;
  static_assert(std::is_same<element_transform_type,
		dataframe::element_transform<T,T>>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_typedef_2,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T,long double>;
  static_assert(std::is_same<element_transform_type,
		dataframe::element_transform<T,long double>>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_typedef_3,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T,std::string>;
  static_assert(std::is_same<element_transform_type,
		dataframe::element_transform<T,std::string>>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_const_typedef,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T const>;
  static_assert(std::is_same<element_transform_type,
		dataframe::element_transform<T const,T const>>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_const_typedef_2,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T const,long double>;
  static_assert(std::is_same<element_transform_type,
		dataframe::element_transform<T const,long double>>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_const_typedef_3,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T const,std::string>;
  static_assert(std::is_same<element_transform_type,
		dataframe::element_transform<T const,std::string>>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_apply,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T>;
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(element_transform_type::apply(t)),T>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(u)),T>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(T(42))),T>::value,"");
  BOOST_CHECK_EQUAL(element_transform_type::apply(t),42);
  BOOST_CHECK_EQUAL(element_transform_type::apply(u),42);
  BOOST_CHECK_EQUAL(element_transform_type::apply(T(42)),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_apply_2,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T,long double>;
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(element_transform_type::apply(t)),long double>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(u)),long double>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(T(42))),long double>::value,"");
  BOOST_CHECK_EQUAL(element_transform_type::apply(t),42);
  BOOST_CHECK_EQUAL(element_transform_type::apply(u),42);
  BOOST_CHECK_EQUAL(element_transform_type::apply(T(42)),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_apply_3,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T,std::string>;
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(element_transform_type::apply(t)),std::string>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(u)),std::string>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(T(42))),std::string>::value,"");
  BOOST_CHECK_EQUAL(element_transform_type::apply(t),"42");
  BOOST_CHECK_EQUAL(element_transform_type::apply(u),"42");
  BOOST_CHECK_EQUAL(element_transform_type::apply(T(42)),"42");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_const_apply,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T const>;
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(element_transform_type::apply(t)),T>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(u)),T>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(T(42))),T>::value,"");
  BOOST_CHECK_EQUAL(element_transform_type::apply(t),42);
  BOOST_CHECK_EQUAL(element_transform_type::apply(u),42);
  BOOST_CHECK_EQUAL(element_transform_type::apply(T(42)),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_const_apply_2,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T const,long double>;
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(element_transform_type::apply(t)),long double>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(u)),long double>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(T(42))),long double>::value,"");
  BOOST_CHECK_EQUAL(element_transform_type::apply(t),42);
  BOOST_CHECK_EQUAL(element_transform_type::apply(u),42);
  BOOST_CHECK_EQUAL(element_transform_type::apply(T(42)),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_const_apply_3,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T const,std::string>;
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(element_transform_type::apply(t)),std::string>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(u)),std::string>::value,"");
  static_assert(std::is_same<decltype(element_transform_type::apply(T(42))),std::string>::value,"");
  BOOST_CHECK_EQUAL(element_transform_type::apply(t),"42");
  BOOST_CHECK_EQUAL(element_transform_type::apply(u),"42");
  BOOST_CHECK_EQUAL(element_transform_type::apply(T(42)),"42");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_functor,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T>;
  element_transform_type transform;  
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(transform(t)),T>::value,"");
  static_assert(std::is_same<decltype(transform(u)),T>::value,"");
  static_assert(std::is_same<decltype(transform(T(42))),T>::value,"");
  BOOST_CHECK_EQUAL(transform(t),42);
  BOOST_CHECK_EQUAL(transform(u),42);
  BOOST_CHECK_EQUAL(transform(T(42)),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_functor_2,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T,long double>;
  element_transform_type transform;
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(transform(t)),long double>::value,"");
  static_assert(std::is_same<decltype(transform(u)),long double>::value,"");
  static_assert(std::is_same<decltype(transform(T(42))),long double>::value,"");
  BOOST_CHECK_EQUAL(transform(t),42);
  BOOST_CHECK_EQUAL(transform(u),42);
  BOOST_CHECK_EQUAL(transform(T(42)),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_functor_3,T,test_types)
{
  using element_transform_type = dataframe::element_transform<T,std::string>;
  element_transform_type transform;
  T t = { 42 };
  T const u = { 42 };
  static_assert(std::is_same<decltype(transform(t)),std::string>::value,"");
  static_assert(std::is_same<decltype(transform(u)),std::string>::value,"");
  static_assert(std::is_same<decltype(transform(T(42))),std::string>::value,"");
  BOOST_CHECK_EQUAL(transform(t),"42");
  BOOST_CHECK_EQUAL(transform(u),"42");
  BOOST_CHECK_EQUAL(transform(T(42)),"42");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_transform_iterator,T,test_types)
{
  using transform = dataframe::element_transform_iterator<T>;
  using vector_type = std::vector<T>;

  vector_type vector = { 42 };

  BOOST_CHECK_EQUAL(*transform::apply(vector.begin()),42);
  BOOST_CHECK_EQUAL(*transform()(vector.begin()),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_transform_iterator_2,T,test_types)
{
  using transform = dataframe::element_transform_iterator<T,long double>;
  using vector_type = std::vector<T>;

  vector_type vector = { 42 };

  BOOST_CHECK_EQUAL(*transform::apply(vector.begin()),42);
  BOOST_CHECK_EQUAL(*transform()(vector.begin()),42);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_transform_iterator_3,T,test_types)
{
  using transform = dataframe::element_transform_iterator<T,std::string>;
  using vector_type = std::vector<T>;

  vector_type vector = { 42 };
  BOOST_CHECK_EQUAL(*transform::apply(vector.begin()),"42");
  BOOST_CHECK_EQUAL(*transform()(vector.begin()),"42");
}

