/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE sample_space_math_multinomial_coefficient

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <sample_space/math/multinomial_coefficient.hpp>
#include <sample_space/vector/vector.hpp>
#include <traits/vector.hpp>
#include <cassert>
#include <boost/math/special_functions/binomial.hpp>
#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector<
  boost::mpl::pair<float,std::vector<float>>
  , boost::mpl::pair<double,std::vector<double>>
  , boost::mpl::pair<long double,std::vector<long double>>
  , boost::mpl::pair<float,sample_space::vector::vector<int>>
  , boost::mpl::pair<double,sample_space::vector::vector<float>>
  , boost::mpl::pair<long double,sample_space::vector::vector<long double>>
  , boost::mpl::pair<double,sample_space::vector::vector<long long>>
  >;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_multinomial_coefficient,Pair,test_types)
{
  using value_type = typename Pair::first;
  using vector_type = typename Pair::second;

  using boost::math::binomial_coefficient;
  using sample_space::math::make_multinomial_coefficient;
  
  vector_type vector = { 1, 2, 3, 4 };
  
  auto expected
    = binomial_coefficient<value_type>(10,1)
    * binomial_coefficient<value_type>(9,2)
    * binomial_coefficient<value_type>(7,3)
    * binomial_coefficient<value_type>(4,4);

  auto multinomial_coefficient = make_multinomial_coefficient<value_type>(vector);
  BOOST_CHECK_EQUAL(multinomial_coefficient.value(),expected);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_multinomial_coefficient_ratio,Pair,test_types)
{
  using value_type = typename Pair::first;
  using vector_type = typename Pair::second;

  using sample_space::math::make_multinomial_coefficient;
  
  vector_type numerator = { 1, 2, 3, 4 };
  vector_type denominator = { 1, 2, 3, 4 };
  
  auto n = make_multinomial_coefficient<value_type>(numerator);
  auto d = make_multinomial_coefficient<value_type>(denominator);
  BOOST_CHECK_EQUAL(static_cast<value_type>(n/d),1);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_multinomial_coefficient_ratio_2,Pair,test_types)
{
  using value_type = typename Pair::first;
  using vector_type = typename Pair::second;

  using boost::math::binomial_coefficient;
  using sample_space::math::make_multinomial_coefficient;
  
  vector_type numerator = { 1, 2, 3, 4 };
  vector_type denominator = { 1, 2, 3 };

  auto expected_numerator
    = binomial_coefficient<value_type>(10,1)
    * binomial_coefficient<value_type>(9,2)
    * binomial_coefficient<value_type>(7,3)
    * binomial_coefficient<value_type>(4,4);
  auto expected_denominator
    = binomial_coefficient<value_type>(6,1)
    * binomial_coefficient<value_type>(5,2)
    * binomial_coefficient<value_type>(3,3);
  auto expected = value_type(expected_numerator) / expected_denominator;

  auto n = make_multinomial_coefficient<value_type>(numerator);
  auto d = make_multinomial_coefficient<value_type>(denominator);
  BOOST_CHECK_EQUAL(static_cast<value_type>(n/d),expected);
}
